/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Preferences Toolbar 3.
 *
 * The Initial Developer of the Original Code is
 * Manuel Reimer <manuel.reimer@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Manuel Reimer <Manuel.Reimer@gmx.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var goPrefBar = Components.classes["@prefbar.mozdev.org/goprefbar;1"]
                          .getService().wrappedJSObject;

var gURL;
var gName;
var gMD5;
var gDownloader;
var gTempfile;

function onLoad() {
  var url = window.arguments[0];
  var success = parseurl(url);
  if (!success) {
    goPrefBar.msgAlert(window, goPrefBar.GetString("urlimport.properties", "msgurlinvalid"));
    window.close();
    return;
  }

  var fname = document.getElementById("nameField");
  var furl = document.getElementById("urlField");
  var fmd5 = document.getElementById("md5Field");
  fname.value = gName;
  furl.value = gURL;
  if (gMD5) fmd5.value = "MD5";

  var aButton = document.documentElement.getButton("accept");
  aButton.setAttribute("label", goPrefBar.GetString("urlimport.properties", "import"));
  aButton.setAttribute("disabled", true);

  // start timer to re-enable buttons
  var delayInterval = 2000;
  setTimeout(reenableInstallButtons, delayInterval);
}

function reenableInstallButtons() {
  document.documentElement.getButton("accept").setAttribute("disabled", false);
}


function cleanup() {
  if (gDownloader) gDownloader.cancelSave();
  gDownloader = undefined;
  if (gTempfile) gTempfile.remove(false);
  return true;
}


// Called by "Import" button
function doDownload() {
  var dirService = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);
  var tempfile = dirService.get("TmpD", Components.interfaces.nsILocalFile);
  tempfile.append("prefbarurlimport.rdf");
  tempfile.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0600);
  gTempfile = tempfile;

  // update window
  var deck = document.getElementById("deck");
  deck.selectedIndex = 1;
  var aButton = document.documentElement.getButton("accept");
  aButton.setAttribute("disabled", true);
  var info = document.getElementById("importInfo");
  info.value = goPrefBar.GetString("urlimport.properties", "infodownloading");
  var infoicon = document.getElementById("infoicon");
  infoicon.setAttribute("class", "message-icon");

  //new obj_URI object
  var obj_URI  = Components.classes['@mozilla.org/network/standard-url;1'].createInstance(Components.interfaces.nsIURI);
  obj_URI.spec = gURL;

  //new persitence object
  gDownloader = Components.classes["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"].createInstance(Components.interfaces.nsIWebBrowserPersist);
  gDownloader.progressListener = new PersistProgressListener;

  //save file to target
  gDownloader.saveURI(obj_URI,null,null,null,null,gTempfile);

  return false;
}

// Called by ProgressListener after finishing download
function downloadDone(status) {
  var info = document.getElementById("importInfo");
  var progress = document.getElementById("progress");
  progress.mode = "undetermined";
  progress.value = 0;

  if (status > 0) {
    goPrefBar.msgAlert(window, goPrefBar.GetString("urlimport.properties", "msgdownloadfailed"));
    cleanup();
    window.close();
    return false;
  }

  if (gMD5) {
    var valid = checkMD5(gTempfile, gMD5);
    if (!valid) {
      goPrefBar.msgAlert(window, goPrefBar.GetString("urlimport.properties", "msgmd5failed"));
      cleanup();
      window.close();
      return false;
    }
  }

  info.value = goPrefBar.GetString("urlimport.properties", "infoimporting");
  var fpService = Components.classes["@mozilla.org/network/protocol;1?name=file"].getService(Components.interfaces.nsIFileProtocolHandler);
  var furl = fpService.getURLSpecFromFile(gTempfile);
  goPrefBar.ImpExp.Import(window, furl);

  goPrefBar.msgAlert(window, goPrefBar.GetString("urlimport.properties", "msgsuccess"));
  cleanup();
  window.close();
}

function checkMD5(file, md5) {
  var istream = Components.classes["@mozilla.org/network/file-input-stream;1"]
    .createInstance(Components.interfaces.nsIFileInputStream);
  // open for reading
  istream.init(file, 0x01, 0444, 0);
  var ch = Components.classes["@mozilla.org/security/hash;1"]
                     .createInstance(Components.interfaces.nsICryptoHash);
  // we want to use the MD5 algorithm
  ch.init(ch.MD5);
  // this tells updateFromStream to read the entire file
  const PR_UINT32_MAX = 0xffffffff;
  ch.updateFromStream(istream, PR_UINT32_MAX);

  // pass false here to get binary data back
  var hash = ch.finish(false);
  // Close stream now, so we are ableo to kill the temporary file on cleanup
  istream.close();

  // convert the binary hash data to a hex string.
  var s = [toHexString(hash.charCodeAt(i)) for (i in hash)].join("");

  if (s.toLowerCase() == md5.toLowerCase()) return true;
  return false;
}
function toHexString(charCode) {
  return ("0" + charCode.toString(16)).slice(-2);
}

function parseurl(url) {
  if (url.toLowerCase().substr(0, 10) != "prefbar://") return false;

  url = url.substr(10);

  var urlparts = url.split("/");

  var lastindex = urlparts.length - 1;
  while(lastindex >= 0) {
    if (urlparts[lastindex].match(/[^\n\t\r ]/)) break;
    urlparts.splice(lastindex, 1);
    lastindex--;
  }

  if (urlparts.length < 4) return false;

  if (urlparts[lastindex].substr(0, 4).toLowerCase() == "md5:") {
    gMD5 = urlparts[lastindex].substr(4);
    urlparts.splice(lastindex, 1);
  }

  lastindex = urlparts.length - 1;
  while(lastindex >= 0) {
    if (urlparts[lastindex].match(/[^\n\t\r ]/)) break;
    urlparts.splice(lastindex, 1);
    lastindex--;
  }

  if (urlparts.length < 4) return false;

  gName = urlparts[lastindex];

  gURL = urlparts.join("/");

  return true;
}

function PersistProgressListener() {}
PersistProgressListener.prototype = {
  QueryInterface : function(aIID) {
    if(aIID.equals(Components.interfaces.nsIWebProgressListener))
      return this;
    throw Components.results.NS_NOINTERFACE;
  },

  init : function() {},
  destroy : function() {},

  onStateChange : function(aWebProgress, aRequest, aStateFlags, aStatus) {
    if (aStateFlags & Components.interfaces.nsIWebProgressListener.STATE_STOP) {
      downloadDone(aStatus);
    }
  },

  onProgressChange : function (aWebProgress, aRequest,
                               aCurSelfProgress, aMaxSelfProgress,
                               aCurTotalProgress, aMaxTotalProgress) {
    var progress = document.getElementById("progress");
    if (aMaxTotalProgress == -1) {
      progress.mode = "undetermined";
    }
    else {
      progress.value = aCurTotalProgress / (aMaxTotalProgress / 100)
    }
  },
  onLocationChange : function(aWebProgress, aRequest, aLocation) {},
  onStatusChange : function(aWebProgress, aRequest, aStatus, aMessage) {},
  onSecurityChange : function(aWebProgress, aRequest, aState) {}
}
